<?php

namespace Drupal\commerce_payu_webcheckout\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\commerce_order\Entity\Order;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Drupal\commerce_payment\Entity\Payment;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\commerce_payu_webcheckout\Entity\Hash;

/**
 * Controller class for path commerce_payu_webcheckout.confirmation.
 */
class ConfirmationController extends ControllerBase {

  const PAYU_SUCCESSFUL = '4';

  private $requestStack;
  protected $moduleHandler;
  private $gatewayConfiguration;

  /**
   * {@inheritdoc}
   */
  public function __construct(RequestStack $request_stack, ModuleHandler $module_handler) {
    $this->requestStack = $request_stack;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack'),
      $container->get('module_handler')
    );
  }

  /**
   * Creates appropiate response for confirmation page.
   *
   * @param Order $commerce_order
   *   The order that has had it's payment handled.
   *
   * @return array
   *   The render array for the page.
   */
  public function response(Order $commerce_order) {
    // If request is not coming from POST, display Access denied.
    $request = $this->requestStack->getCurrentRequest();
    if ($request->getMethod() != 'POST') {
      throw new AccessDeniedHttpException();
    }
    \Drupal::logger('log')->notice('Flag 1');
    $parameters = $request->request->all();
    // If no hash provided, display Access denied.
    if (!isset($parameters['sign'])) {
      throw new AccessDeniedHttpException();
    }
    \Drupal::logger('log')->notice('Flag 2');
    // If calculated hash does not coincide, display Access denied.
    if (!$this->validateIncomingHash($parameters['sign'], $parameters, $commerce_order)) {
      throw new AccessDeniedHttpException();
    }
    \Drupal::logger('log')->notice('Flag 3');
    // If stored hash does not coincide, display access denied.
    if (!$this->validateStoredHash($parameters, $commerce_order)) {
      throw new AccessDeniedHttpException();
    }
    \Drupal::logger('log')->notice('Flag 4');
    if ($parameters['state_pol'] != self::PAYU_SUCCESSFUL) {
      // Payment has not been successful.
      $this->createUnsucessfulPayment($parameters, $commerce_order);
    }
    else {
      // Payment has been successful.
      $this->createSuccessfulPayment($parameters, $commerce_order);
    }
    return new JsonResponse(array('status' => 'OK'));
  }

  /**
   * Validates whether the stored hash is valid.
   *
   * @param array $parameters
   *   The set of parameters in request query.
   * @param Drupal\commerce_order\Entity\Order $commerce_order
   *   The current order.
   *
   * @return bool
   *   Whether the stored hash matched information contained in parameter bag.
   */
  private function validateStoredHash($parameters, Order $commerce_order) {
    $configuration = $this->retrieveGatewayConfiguration($commerce_order);
    $hash = Hash::createHash($configuration['api_key'], $parameters['merchant_id'], $parameters['reference_sale'], $parameters['value'], $parameters['currency']);
    $hashes = $this->entityTypeManager()->getStorage('payu_hash')->loadByProperties(array(
      'hash' => $hash,
    ));
    if (!$hashes) {
      return FALSE;
    }
    $hash = reset($hashes);
    // For added security, we check whether the Order and User ID match.
    $order = $hash->getOrder();
    $user = $hash->getUser();
    if ($commerce_order->getCustomer()->id() == $user->id() && $order->getOrderNumber() == $commerce_order->getOrderNumber()) {
      // There is a match. We delete the hash if state pol is successful.
      if ($parameters['state_pol'] == self::PAYU_SUCCESSFUL) {
        $hash->delete();
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Validates whether the incoming hash is valid.
   *
   * @param string $incoming_hash
   *   The incoming hash.
   * @param array $parameters
   *   The set of parameters in request query.
   * @param Drupal\commerce_order\Entity\Order $commerce_order
   *   The order.
   *
   * @return bool
   *   Whether the incoming hash is valid.
   */
  private function validateIncomingHash($incoming_hash, $parameters, Order $commerce_order) {
    $configuration = $this->retrieveGatewayConfiguration($commerce_order);
    // Round Half Even rounding in order to comply with Payu's specs.
    $amount = $parameters['value'];
    $amount = round($amount, 1, PHP_ROUND_HALF_EVEN);
    if (($amount - (int) $amount) == 0) {
      $amount = $amount . '.0';
    }
    $hash = Hash::createHash($configuration['api_key'], $parameters['merchant_id'], $parameters['reference_sale'], $amount, $parameters['currency'], $parameters['state_pol']);
    return $incoming_hash == $hash;
  }

  /**
   * Lazy loads gatewayConfiguration property.
   *
   * @param Drupal\commerce_order\Entity\Order $commerce_order
   *   The order.
   *
   * @return array
   *   Payment gateway plugin configuration.
   */
  private function retrieveGatewayConfiguration(Order $commerce_order) {
    if ($this->gatewayConfiguration) {
      return $this->gatewayConfiguration;
    }
    $payment_gateway = $commerce_order->payment_gateway->entity;
    $payment_gateway_plugin = $payment_gateway->getPlugin();
    $this->gatewayConfiguration = $payment_gateway_plugin->getConfiguration();
    return $this->gatewayConfiguration;
  }

  /**
   * Creates an unsuccessful payment by adjusting the transition to the entity.
   *
   * @param array $parameters
   *   The set of parameters in request query.
   * @param Drupal\commerce_order\Entity\Order $commerce_order
   *   The order.
   */
  private function createUnsucessfulPayment($parameters, Order $commerce_order) {
    $payment = $this->createPayment($parameters, $commerce_order);
    $transitions = $payment->getState()->getWorkflow()->getTransitions();
    $payment->getState()->applyTransition($transitions['void']);
    $payment->save();
  }

  /**
   * Creates a successful payment by adjusting the transition to the entity.
   *
   * @param array $parameters
   *   The set of parameters in request query.
   * @param Drupal\commerce_order\Entity\Order $commerce_order
   *   The order.
   */
  private function createSuccessfulPayment($parameters, Order $commerce_order) {
    $payment = $this->createPayment($parameters, $commerce_order);
    $transitions = $payment->getState()->getWorkflow()->getTransitions();
    $payment->getState()->applyTransition($transitions['complete']);
    $payment->save();
    // We move the order status to the next state as well.
    $transitions = $commerce_order->getState()->getWorkflow()->getTransitions();
    $new_transition = 'place';
    // Allow developers to modify new transition at will.
    $this->moduleHandler->alter('payu_transition_after_payment', $new_transition);
    if (!in_array($new_transition, array_keys($transitions))) {
      throw new \InvalidArgumentException('The transition ' . $new_transition . ', does not exist.');
    }
    $commerce_order->getState()->applyTransition($transitions[$new_transition]);
    $commerce_order->save();
  }

  /**
   * Helper that creates a new Payment entity regardless of success status.
   *
   * @param array $parameters
   *   The set of parameters in request query.
   * @param Drupal\commerce_order\Entity\Order $commerce_order
   *   The order.
   *
   * @return Drupal\commerce_payment\Entity\Payment
   *   The new Payment entity.
   */
  private function createPayment($parameters, Order $commerce_order) {
    return Payment::create(
      array(
        'type' => 'payu_webcheckout_payment',
        'payment_method' => $commerce_order->payment_method->target_id,
        'payment_gateway' => $commerce_order->payment_gateway->target_id,
        'order_id' => $commerce_order->id(),
        'amount' => $commerce_order->getTotalPrice(),
        'remote_id' => $parameters['transaction_id'],
        'cc_number' => $parameters['cc_number'],
        'cc_holder' => $parameters['cc_holder'],
        'transaction_date' => $parameters['transaction_date'],
        'response_message_pol' => ucfirst(strtolower(str_replace('_', ' ', $parameters['response_message_pol']))),
        'error_message_bank' => ucfirst(strtolower(str_replace('_', ' ', $parameters['error_message_bank']))),
        'payment_method_name' => $parameters['payment_method_name'],
        'pse_bank' => $parameters['pse_bank'],
        'currency' => $parameters['currency'],
        'exchange_rate' => $parameters['exchange_rate'],
      )
    );
  }

}
