<?php

namespace Drupal\commerce_payu_webcheckout\Plugin\Commerce\CheckoutPane;

use Drupal\commerce_checkout\Plugin\Commerce\CheckoutFlow\CheckoutFlowInterface;
use Drupal\commerce_checkout\Plugin\Commerce\CheckoutPane\CheckoutPaneBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManager;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\commerce_payu_webcheckout\Entity\Hash;

/**
 * Provides the Payu WebCheckout payment pane.
 *
 * @CommerceCheckoutPane(
 *   id = "payu_webcheckout_payment_pane",
 *   label = @Translation("Payu WebCheckout"),
 *   admin_label = @Translation("Payu Webcheckout pane"),
 *   default_step = "offsite_payment",
 *   wrapper_element = "container",
 * )
 */
class PayuWebCheckoutPane extends CheckoutPaneBase implements ContainerFactoryPluginInterface {

  private $languageManager;
  private $urlGenerator;

  const TEST_API_KEY = '4Vj8eK4rloUd272L48hsrarnUA';
  const TEST_MERCHANT_ID = '508029';
  const TEST_ACCOUNT_ID = '512321';
  const TEST_GATEWAY_URL = 'https://sandbox.gateway.payulatam.com/ppp-web-gateway/';
  const PROD_GATEWAY_URL = 'https://gateway.payulatam.com/ppp-web-gateway/';

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, CheckoutFlowInterface $checkout_flow, LanguageManager $languageManager, $url_generator) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $checkout_flow);
    $this->languageManager = $languageManager;
    $this->urlGenerator = $url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, CheckoutFlowInterface $checkout_flow = NULL) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $checkout_flow,
      $container->get('language_manager'),
      $container->get('url_generator')
    );
  }

  /**
   * Alters the pane form so it adds PayU specific fields.
   *
   * @param array &$form
   *   The form being modified.
   */
  private function buildPayuForm(array &$form) {
    $payment_gateway = $this->order->payment_gateway->entity;
    if (!$payment_gateway) {
      return '';
    }
    $payment_gateway_plugin = $payment_gateway->getPlugin();
    $configuration = $payment_gateway_plugin->getConfiguration();
    $is_test = $configuration['mode'] == 'test' ? TRUE : FALSE;
    $billing_address = $this->order->getBillingProfile()->get('address')->getValue();
    $billing_address = reset($billing_address);
    $api_key = $is_test ? self::TEST_API_KEY : $configuration['api_key'];
    $merchant_id = $is_test ? self::TEST_MERCHANT_ID : $configuration['merchant_id'];
    $account_id = $is_test ? self::TEST_ACCOUNT_ID : $configuration['account_id'];
    $reference_code = 'PAYULATAM-' . $this->order->getOrderNumber() . '-' . time();
    $amount = money_format('%!.2n', $this->order->getTotalPrice()->getNumber());
    $currency_code = $this->order->getTotalPrice()->getCurrencyCode();
    $owner = $this->order->getCustomer();
    $hash = Hash::create(
      array(
        'hash' => Hash::createHash($api_key, $merchant_id, $reference_code, $amount, $currency_code),
        'created' => time(),
        'uid' => $owner->id(),
        'order_id' => $this->order->getOrderNumber(),
      )
    );
    $hash->save();
    $order_information = array(
      'merchantId' => $merchant_id,
      'accountId' => $account_id,
      'description' => $configuration['order_description'],
      'referenceCode' => $reference_code,
      'amount' => $amount,
      'tax' => '0',
      'taxReturnBase' => '0',
      'currency' => $currency_code,
      'signature' => $hash->getHash(),
      'buyerEmail' => $this->order->getEmail(),
      'responseUrl' => $this->urlGenerator->generateFromRoute('commerce_checkout.form', array(
        'commerce_order' => $this->order->getOrderNumber(),
        'step' => $this->checkoutFlow->getNextStepId(),
      ), array(
        'absolute' => TRUE,
      )),
      'confirmationUrl' => $this->urlGenerator->generateFromRoute('commerce_payu_webcheckout.confirmation', array(
        'commerce_order' => $this->order->getOrderNumber(),
      ), array(
        'absolute' => TRUE,
      )),
      'lng' => ($owner->language) ? $owner->language : $this->languageManager->getDefaultLanguage()->getId(),
    );

    if ($is_test) {
      $order_information['test'] = '1';
    }

    if ($billing_address['address_line1']) {
      $order_information['billingAddress'] = $billing_address['address_line1'];
    }

    if ($billing_address['locality']) {
      $order_information['billingCity'] = $billing_address['locality'];
    }

    if ($billing_address['postal_code']) {
      $order_information['zipCode'] = $billing_address['postal_code'];
    }

    if ($billing_address['country_code']) {
      $order_information['billingCountry'] = $billing_address['country_code'];
    }

    foreach ($order_information as $order_item_key => $order_item_Value) {
      $form[$order_item_key] = array(
        '#type' => 'hidden',
        '#value' => $order_item_Value,
      );
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildPaneForm(array $pane_form, FormStateInterface $form_state, array &$complete_form) {
    if ($this->order->getState()->getString() == 'completed') {
      $pane_form['#parents'] = array();
      $pane_form['redirection_message'] = array(
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => t('Payment for this order has already been conducted.'),
      );
      return $pane_form;
    }
    $pane_form['#parents'] = array();
    $pane_form['redirection_message'] = array(
      '#type' => 'html_tag',
      '#tag' => 'p',
      '#value' => t('You are being redirected to PayU WebCheckout to continue payment.'),
    );
    $this->buildPayuForm($pane_form);
    $complete_form['#action'] = self::PROD_GATEWAY_URL;
    if (isset($pane_form['test'])) {
      $complete_form['#action'] = self::TEST_GATEWAY_URL;
    }
    $pane_form['#attached']['library'][] = 'commerce_payu_webcheckout/payu.loader';
    return $pane_form;
  }

}
