<?php

namespace Drupal\commerce_payu_webcheckout\Plugin\Commerce\PaymentMethodType;

use Drupal\commerce_payment\Plugin\Commerce\PaymentMethodType\PaymentMethodTypeBase;
use Drupal\commerce_payment\Entity\PaymentMethodInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\commerce_checkout\CheckoutOrderManagerInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\commerce_payu_webcheckout\Plugin\Commerce\PaymentGateway\PayuWebcheckoutGateway;
use Drupal\Core\Discovery\YamlDiscovery;
use Drupal\Core\Render\RendererInterface;

/**
 * Provides the Payu WebCheckout payment method type.
 *
 * @CommercePaymentMethodType(
 *   id = "payu_webcheckout_payment_type",
 *   label = @Translation("Payu Webcheckout"),
 *   create_label = @Translation("Pay using PayU WebCheckout."),
 * )
 */
class PayuWebcheckoutMethodType extends PaymentMethodTypeBase implements ContainerFactoryPluginInterface {

  private $currentRouteMatch;
  private $checkoutOrderManager;
  private $entityTypeManager;
  private $renderer;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, CurrentRouteMatch $current_route_match, CheckoutOrderManagerInterface $checkout_order_manager, EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->currentRouteMatch = $current_route_match;
    $this->checkoutOrderManager = $checkout_order_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_route_match'),
      $container->get('commerce_checkout.checkout_order_manager'),
      $container->get('entity_type.manager'),
      $container->get('renderer')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildLabel(PaymentMethodInterface $payment_method) {
    return 'PayU WebCheckout';
  }

  /**
   * Loads the gateway plugin for the current order.
   *
   * @return PayuWebcheckoutGateway
   *   The Plugin gateway.
   */
  private function getCurrentGatewayPlugin() {
    // We load the current commerce_order.
    $order = $this->currentRouteMatch->getParameter('commerce_order');
    if (!$order) {
      return FALSE;
    }
    $payment_gateway = $order->payment_gateway->entity;
    if ($payment_gateway) {
      // This order was already attempted. We can go ahead and retrieve
      // the gateway plugin.
      return $payment_gateway->getPlugin();
    }
    else {
      // We need to load the gateway as Commerce is doing:
      $checkout_flow_plugin = $order->checkout_flow->entity->getPlugin();
      $checkout_flow_order = $checkout_flow_plugin->getOrder();
      $payment_gateway_storage = $this->entityTypeManager->getStorage('commerce_payment_gateway');
      $payment_gateways = $payment_gateway_storage->loadMultipleForOrder($checkout_flow_order);
      if (empty($payment_gateways)) {
        return FALSE;
      }
      $gateway = reset($payment_gateways);
      return $gateway->getPlugin();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCreateLabel() {
    $gateway = $this->getCurrentGatewayPlugin();
    if (!$gateway instanceof PayuWebcheckoutGateway) {
      return;
    }
    $selection = $gateway->getImageConfiguration();
    if (!$selection || $selection == 'no_image') {
      return $this->pluginDefinition['create_label'];
    }
    $path = drupal_get_path('module', 'commerce_payu_webcheckout');
    $discovery = new YamlDiscovery('images', array('commerce_payu_webcheckout' => $path));
    $definitions = $discovery->findAll();
    $definitions = reset($definitions);

    $content = array(
      '#theme' => 'payu_webcheckout_payment_method',
      '#payu_html' => $definitions['payu_webcheckout_images'][$selection]['html'],
    );
    return $this->renderer->render($content);
  }

}
